"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _os = require("os");

const destroy = async (pool, options) => {
  const {
    executeStatement,
    connect,
    RDSDataService,
    escapeId,
    escape,
    fullJitter,
    coercer,
    dispose
  } = pool;
  const admin = {
    config: {
      awsSecretStoreArn: options.awsSecretStoreAdminArn,
      dbClusterOrInstanceArn: options.dbClusterOrInstanceArn,
      databaseName: options.databaseName,
      tableName: options.tableName,
      region: options.region
    }
  };
  await connect(admin, {
    RDSDataService,
    escapeId,
    escape,
    fullJitter,
    executeStatement,
    coercer
  });
  let alterSchemaError = null;
  let dropSchemaError = null;

  try {
    await executeStatement(admin, `ALTER SCHEMA ${escapeId(options.databaseName)} OWNER TO SESSION_USER`);
  } catch (error) {
    alterSchemaError = error;
  }

  try {
    await executeStatement(admin, `DROP SCHEMA ${escapeId(options.databaseName)} CASCADE`);
  } catch (error) {
    dropSchemaError = error;
  }

  if (alterSchemaError != null || dropSchemaError != null) {
    const error = new Error();
    error.message = `${alterSchemaError != null ? `${alterSchemaError.message}${_os.EOL}` : ''}${dropSchemaError != null ? `${dropSchemaError.message}${_os.EOL}` : ''}`;
    throw error;
  }

  await dispose(admin);
};

var _default = destroy;
exports.default = _default;
//# sourceMappingURL=destroy.js.map